@@ -21,17 +21,20 @@
 
 #include "ev-application.h"
 #include "ev-metadata-manager.h"
+#include "ev-debug.h"
 
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
 #include <stdlib.h>
 #include <string.h>
+
+#if WITH_GNOME
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-app-helper.h>
 #include <libgnomeui/gnome-authentication-manager.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
+#endif
 
 #ifdef ENABLE_DBUS
 #include <dbus/dbus-glib-bindings.h>
@@ -40,12 +43,15 @@
 #include "ev-stock-icons.h"
 #include "ev-job-queue.h"
 #include "ev-file-helpers.h"
+#include "ev-backends-manager.h"
 
 static gchar   *ev_page_label;
+static gchar   *ev_find_string;
 static gboolean preview_mode = FALSE;
 static gboolean fullscren_mode = FALSE;
 static gboolean presentation_mode = FALSE;
 static gboolean unlink_temp_file = FALSE;
+static gchar   *print_settings;
 static const char **file_arguments = NULL;
 
 static const GOptionEntry goption_options[] =
@@ -54,7 +60,9 @@ static const GOptionEntry goption_options[] =
 	{ "fullscreen", 'f', 0, G_OPTION_ARG_NONE, &fullscren_mode, N_("Run evince in fullscreen mode"), NULL },
 	{ "presentation", 's', 0, G_OPTION_ARG_NONE, &presentation_mode, N_("Run evince in presentation mode"), NULL },
 	{ "preview", 'w', 0, G_OPTION_ARG_NONE, &preview_mode, N_("Run evince as a previewer"), NULL },
-	{ "unlink-temp-file", 'u', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &unlink_temp_file, NULL, NULL },
+	{ "find", 'l', 0, G_OPTION_ARG_STRING, &ev_find_string, N_("The word or phrase to find in the document"), N_("STRING")},
+	{ "unlink-tempfile", 'u', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &unlink_temp_file, NULL, NULL },
+	{ "print-settings", 't', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME, &print_settings, NULL, NULL },
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &file_arguments, NULL, N_("[FILE...]") },
 	{ NULL }
 };
@@ -66,6 +74,24 @@ value_free (GValue *value)
 	g_free (value);
 }
 
+/**
+ * arguments_parse:
+ *
+ * Parses the arguments and creates a #GHashTable with this data.
+ *
+ *  key                 ->  value
+ *
+ *  dislay              ->  display at the default screen.
+ *  screen              ->  screen number.
+ *  page-label          ->  only if the page label argument has been passed,
+ *                          the page of the document to display.
+ *  mode                ->  only if the view mode is one of the availables,
+ *                          the view mode.
+ *  unlink-temp-file    ->  only if the view mode is preview mode and
+ *                          unlink-temp-file has been passed, unlink-temp-file.
+ *
+ * Returns: a pointer into #GHashTable with data from the arguments.
+ */
 static GHashTable *
 arguments_parse (void)
 {
@@ -104,6 +130,20 @@ arguments_parse (void)
 		g_value_set_string (value, ev_page_label);
 
 		g_hash_table_insert (args, g_strdup ("page-label"), value);
+
+		g_free (ev_page_label);
+		ev_page_label = NULL;
+	}
+
+	if (ev_find_string) {
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, ev_find_string);
+
+		g_hash_table_insert (args, g_strdup ("find-string"), value);
+
+		g_free (ev_find_string);
+		ev_page_label = NULL;
 	}
 
 	if (fullscren_mode)
@@ -131,6 +171,18 @@ arguments_parse (void)
 				     value);
 	}
 
+	if (mode == EV_WINDOW_MODE_PREVIEW && print_settings) {
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_STRING);
+		g_value_set_string (value, print_settings);
+
+		g_hash_table_insert (args,
+				     g_strdup ("print-settings"),
+				     value);
+		g_free (print_settings);
+		print_settings = NULL;
+	}
+
 	return args;
 }
 
@@ -149,10 +201,13 @@ load_files (const char **files,
 		char   *uri;
 		char   *label;
 		GValue *old = NULL;
+		GFile  *file;
 
-		uri = gnome_vfs_make_uri_from_shell_arg (files[i]);
+		file = g_file_new_for_commandline_arg (files[i]);
+		uri = g_file_get_uri (file);
+		g_object_unref (file);
 		
-		label = strchr (uri, GNOME_VFS_URI_MAGIC_CHR);
+		label = strchr (uri, '#');
 
 		if (label) {
 			GValue *new;
@@ -197,7 +252,7 @@ load_files_remote (const char **files,
 	connection = dbus_g_bus_get (DBUS_BUS_STARTER, &error);
 
 	if (connection == NULL) {
-		g_warning (error->message);
+		g_warning ("%s", error->message);
 		g_error_free (error);	
 
 		return FALSE;
@@ -213,7 +268,7 @@ load_files_remote (const char **files,
 					G_TYPE_UINT, timestamp,
 					G_TYPE_INVALID,
 					G_TYPE_INVALID)) {
-			g_warning (error->message);
+			g_warning ("%s", error->message);
 			g_clear_error (&error);
 			g_object_unref (remote_object);
 			dbus_g_connection_unref (connection);
@@ -228,9 +283,13 @@ load_files_remote (const char **files,
 
 	for (i = 0; files[i]; i++) {
 		const char *page_label;
+		GFile *file;
 		char *uri;
 
-		uri = gnome_vfs_make_uri_from_shell_arg (files[i]);
+		file = g_file_new_for_commandline_arg (files[i]);
+		uri = g_file_get_uri (file);
+		g_object_unref (file);
+
 		page_label = ev_page_label ? ev_page_label : "";
 
 		if (!dbus_g_proxy_call (remote_object, "OpenURI", &error,
@@ -239,7 +298,7 @@ load_files_remote (const char **files,
 					G_TYPE_UINT, timestamp,
 					G_TYPE_INVALID,
 					G_TYPE_INVALID)) {
-			g_warning (error->message);
+			g_warning ("%s", error->message);
 			g_clear_error (&error);
 			g_free (uri);
 			continue;
@@ -264,7 +323,12 @@ main (int argc, char *argv[])
 	gboolean enable_metadata = FALSE;
 	GOptionContext *context;
 	GHashTable *args;
+#if WITH_GNOME
 	GnomeProgram *program;
+#else
+	char *accel_filename;
+	GError *error = NULL;
+#endif
 
 	context = g_option_context_new (_("GNOME Document Viewer"));
 
@@ -274,16 +338,30 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 	g_option_context_add_main_entries (context, goption_options, GETTEXT_PACKAGE);
+	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
 #else
 	g_option_context_add_main_entries (context, goption_options, NULL);
 #endif
 
+#if WITH_GNOME
 	program = gnome_program_init (PACKAGE, VERSION,
-                                      LIBGNOMEUI_MODULE, argc, argv,
-                                      GNOME_PARAM_GOPTION_CONTEXT, context,
-                                      GNOME_PARAM_HUMAN_READABLE_NAME, _("Evince"),
+				      LIBGNOMEUI_MODULE, argc, argv,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
+				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Evince"),
 				      GNOME_PARAM_APP_DATADIR, GNOMEDATADIR,
-                                      NULL);
+				      NULL);
+#else
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	if (!g_option_context_parse (context, &argc, &argv, &error)) {
+		g_warning ("Cannot parse arguments: %s", error->message);
+		g_error_free (error);
+		return 1;
+	}
+	g_option_context_free (context);
+
+	accel_filename = g_build_filename (ev_dot_dir (), "accels", NULL);
+	gtk_accel_map_load (accel_filename);
+#endif
 
 	args = arguments_parse ();
 
@@ -291,17 +369,24 @@ main (int argc, char *argv[])
 	if (!ev_application_register_service (EV_APP)) {
 		if (load_files_remote (file_arguments, args)) {
 			g_hash_table_destroy (args);
+#if WITH_GNOME
 			g_object_unref (program);
-			
+#endif
 			return 0;
 		}
 	} else {
 		enable_metadata = TRUE;
 	}
 #endif
-	
-	gnome_authentication_manager_init ();
 
+	ev_debug_init ();
+	
+#if WITH_GNOME	
+	gnome_authentication_manager_init ();
+#endif
+
+	ev_backends_manager_init ();
+	
 	if (enable_metadata) {
 		ev_metadata_manager_init ();
 	}
@@ -318,13 +403,24 @@ main (int argc, char *argv[])
 
 	gtk_main ();
 
+#if WITH_GNOME
 	gnome_accelerators_sync ();
+#else
+	gtk_accel_map_save (accel_filename);
+	g_free (accel_filename);
+#endif
+
 	ev_file_helpers_shutdown ();
 
 	if (enable_metadata) {
 		ev_metadata_manager_shutdown ();
 	}
- 	g_object_unref (program);
 
+	ev_backends_manager_shutdown ();
+
+#if WITH_GNOME
+ 	g_object_unref (program);
+#endif
+	
 	return 0;
 }
